<?php

declare(strict_types=1);

namespace mirzaev\ebala\controllers;

// Файлы проекта
use mirzaev\ebala\controllers\core,
	mirzaev\ebala\controllers\traits\errors,
	mirzaev\ebala\models\socket as model,
	mirzaev\ebala\models\task,
	mirzaev\ebala\models\session,
	mirzaev\ebala\models\registry,
	mirzaev\ebala\models\core as _core;

// Фреймворк ArangoDB
use mirzaev\arangodb\document;

// Библиотека для ArangoDB
use ArangoDBClient\Document as _document;

// Встроенные библиотеки
use exception;

/**
 * Контроллер сокета
 *
 * @package mirzaev\ebala\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class socket extends core
{
	use errors;

	/**
	 * Регистрация
	 *
	 * @param array $parameters Параметры запроса
	 */
	public function registration(array $parameters = []): ?string
	{
		try {
			if (model::session($this->session->getId(), $parameters['key'], $this->errors)) {
				// Инициализировано соединение
	
				// @todo сделать например возврат того что соединение удалось и на клиенте что-то оптимизировать (или не удалось чтобы повторил)
				return null;
			} else throw new exception('Не удалось зарегистрировать сокет');
		} catch (exception $e) {
			// Write to the errors registry
			$this->errors['socket'][] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Возврат (провал)
		return null;
	}
}
