<?php

declare(strict_types=1);

namespace mirzaev\ebala\models;

// Project files
use mirzaev\ebala\models\traits\instance,
	mirzaev\ebala\models\traits\status,
	mirzaev\ebala\models\account,
	mirzaev\ebala\models\worker,
	mirzaev\ebala\models\market;

// Библиотека для ArangoDB
use ArangoDBClient\Document as _document;

// Фреймворк ArangoDB
use mirzaev\arangodb\collection,
	mirzaev\arangodb\document;

// Встроенные библиотеки
use exception;

/**
 * Модель настроек
 *
 * Управляет записью и чтением настроек сайта из ArangoDB
 *
 * @package mirzaev\ebala\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class settings extends core
{
	use instance, status;

	/**
	 * Collection name in ArangoDB
	 */
	final public const COLLECTION = 'settings';

	/**
	 * Инстанция документа в базе данных 
	 */
	protected readonly _document $document;

	/**
	 * Read (search)
	 *
	 * @param ?string $before Injection of AQL-code before search of edges
	 * @param int $amount Amount of administrators
	 * @param int $page Offset by amount
	 * @param string $target Collection or view name
	 * @param array $binds Binds for query
	 * @param array $errors Errors registry
	 *
	 * @return array Instances from ArangoDB
	 */
	public static function search(
		?string $before = '',
		int $amount = 1000,
		int $page = 1,
		string $sort = 'setting.category ASC, setting.city ASC, setting.work ASC, setting.rating DESC, setting.created DESC, setting._key DESC',
		string $target = settings::COLLECTION,
		array $binds = [],
		array &$errors = []
	): array {
		try {
			if (collection::init(static::$arangodb->session, settings::COLLECTION)) {
				// Инициализирована коллекция

				// Search the session data in ArangoDB
				$settings = collection::search(static::$arangodb->session, sprintf(
					<<<AQL
						FOR setting IN %s
							%s
							SORT %s
							LIMIT %d, %d
							LET d = setting.category
							COLLECT x = setting.category INTO groups 
							RETURN { [x]: groups[*]['setting'] }
					AQL,
					$target,
					$before,
					$sort,
					--$page <= 0 ? 0 : $amount * $page,
					$amount
				), $binds);

				// Универсализация значений
				$buffer = [];
				foreach (is_array($settings) ? $settings : [$settings] as $setting) foreach ($setting->getAll() ?? [] as $category => $data) $buffer[$category] = $data;
				$settings = $buffer;

				// Exit (success)
				return $settings;
			} else throw new exception('Не удалось инициализировать коллекции');
		} catch (exception $e) {
			// Write to the errors registry
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
			var_dump($errors);
		}

		// Exit (fail)
		return [];
	}
}
